#!/usr/bin/env python3
# coding: utf-8
##################################
# #######   ITP Class   ##########
##################################


import os
import logging
import sys

from os_command_py import os_command

from .topmol import TopMol

# Logging
logger = logging.getLogger(__name__)


def show_log():
    """ To use only with Doctest !!!
    Redirect logger output to sys.stdout
    """
    # Delete all handlers
    logger.handlers = []
    # Set the logger level to INFO
    logger.setLevel(logging.INFO)
    # Add sys.sdout as handler
    logger.addHandler(logging.StreamHandler(sys.stdout))

    # Show log of top sys:
    from . import topmol
    topmol.show_log()


# Autorship information
__author__ = "Samuel Murail"
__copyright__ = "Copyright 2020, RPBS"
__credits__ = ["Samuel Murail"]
__license__ = "GNU General Public License v2.0"
__version__ = "1.2.1"
__maintainer__ = "Samuel Murail"
__email__ = "samuel.murail@u-paris.fr"
__status__ = "Prototype"


# Position restraints files:
def write_index_posre_file(atom_index_list, posre_file, type_val=1,
                           fc=[1000, 1000, 1000]):
    """Write a pos restraint file based on atom index list
    """

    filout = open(posre_file, 'w')
    filout.write("; Position restraint file created by " + __name__ + "\n")
    filout.write("[ position_restraints ]\n")
    filout.write("; atom  type      fx      fy      fz\n")

    for index in atom_index_list:
        filout.write("{:6d}{:6d}{:6d}{:6d}{:6d} \n".format(
            index, type_val, fc[0], fc[1], fc[2]))

    filout.write("\n")
    filout.close()


class Itp:
    """Itp topologie in gromacs format
    May contain several molecule definition, so
    itp class is a collection of top_mol object
    which are indivudial molecule topologies
    """

    def __init__(self, name, fullname, path):
        """Read an itp file and extract all the top_mol
        """

        # name is useless to fix, maybe only put shortpath and fullpath
        self.name = name
        self.fullname = fullname
        self.path = path
        self.top_mol_list = []
        self.posres_file = []
        self.read_file()

    def read_file(self):

        field = None
        ifdef = False
        posre_def = ""
        local_top = None
        local_top_flag = False

        with open(self.path) as file:
            for line in file:
                # print("Itp line: \"{}\" ".format(line))
                # Check posres include:
                if line[:6] == '#ifdef':
                    ifdef = True
                    line_list = line.split()
                    posre_def = line_list[1]
                if line[:6] == '#endif':
                    ifdef = False
                if ifdef and line[:8] == '#include':
                    line_list = line.split()
                    posre_file = line_list[1][1:-1]
                    self.posres_file.append(
                        {'def': posre_def, 'file': posre_file})
                if line.strip().startswith("["):
                    # Remove space and [ ], remove also comments
                    field = line.replace(" ", "").split("]")[0][1:]
                    continue
                if line[0] != ";" and line[0] != "#" and line.strip() != "":
                    # Remove commentary in the line
                    line_comment = line.split(';')
                    line_list = line_comment[0].split()
                    # print(line_list)

                    if field == 'moleculetype':
                        # Check if a top_mol already exist, if yes append it
                        # to the top_mol_list
                        if local_top_flag:
                            # print("Add mol topologie",local_top.name)
                            self.top_mol_list.append(local_top)
                        # Create a new top_mol
                        local_top = TopMol(line_list[0], line_list[1])
                        local_top_flag = True

                    elif field == 'atoms':
                        atom_num = int(line_list[0])
                        atom_type = line_list[1]
                        # In rare cases the residue num is not an integer
                        # With duplicate residues like : eg "184A"
                        try:
                            res_num = int(line_list[2])
                        except ValueError:
                            res_num = line_list[2]
                        res_name = line_list[3]
                        atom_name = line_list[4]
                        charge_num = int(line_list[5])
                        charge = float(line_list[6])
                        if len(line_list) > 7:
                            mass = float(line_list[7])
                        else:
                            mass = None
                        atom = {"num": atom_num,
                                "atom_type": atom_type,
                                "atom_name": atom_name,
                                "res_num": res_num,
                                "res_name": res_name,
                                "charge_num": charge_num,
                                "charge": charge,
                                "mass": mass}
                        local_top.atom_dict[atom_num] = atom
                    # Read only indexes not the parameters (c0, c1 ...)
                    # May need some modifications :
                    elif field == 'bonds':
                        # for i in range(3):
                        #    print("val:{} #{}#".format(i, line[i*5:(i+1)*5]))
                        # ai, aj, funct = [int(line[i*5:(i+1)*5]) for i in
                        # range(3)]
                        ai, aj, funct = [int(col) for col in line_list[:3]]
                        if len(line_list) < 5:
                            r = k = ''
                        else:
                            r, k = line_list[3:5]
                        local_top.bond_list.append({'ai': ai, 'aj': aj,
                                                    'funct': funct, 'r': r,
                                                    'k': k})
                    elif field == 'constraints':
                        ai, aj, funct = [int(col) for col in line_list[:3]]
                        local_top.cons_list.append({'ai': ai, 'aj': aj,
                                                    'funct': funct})
                    elif field == 'pairs':
                        ai, aj, funct = [int(col) for col in line_list[:3]]
                        local_top.pair_list.append({'ai': ai, 'aj': aj,
                                                    'funct': funct})
                    elif field == 'angles':
                        ai, aj, ak, funct = [int(col) for col in line_list[:4]]
                        if len(line_list) < 6:
                            theta = cth = ''
                        else:
                            theta, cth = line_list[4:6]
                        local_top.angl_list.append({'ai': ai, 'aj': aj,
                                                    'ak': ak, 'funct': funct,
                                                    'theta': theta,
                                                    'cth': cth})
                    elif field == 'dihedrals':
                        ai, aj, ak, al, funct = [int(col) for col in
                                                 line_list[:5]]
                        if len(line_list) < 7:
                            phase = kd = pn = ''
                        else:
                            phase, kd = line_list[5:7]
                            pn = int(line_list[7])
                        local_top.dihe_list.append({'ai': ai, 'aj': aj,
                                                    'ak': ak, 'al': al,
                                                    'funct': funct,
                                                    'phase': phase, 'kd': kd,
                                                    'pn': pn})
                    elif field == 'virtual_sites3':
                        ai, aj, ak, al, funct = [
                            int(col) for col in line_list[:5]]
                        local_top.vs3_list.append({'ai': ai, 'aj': aj,
                                                   'ak': ak, 'al': al,
                                                   'funct': funct})
                    elif field == 'cmap':
                        ai, aj, ak, al, am, funct = [
                            int(col) for col in line_list[:6]]
                        local_top.cmap_list.append({'ai': ai, 'aj': aj,
                                                    'ak': ak, 'al': al,
                                                    'am': am, 'funct': funct})
                    elif field == 'virtual_sites4':
                        ai, aj, ak, al, am, funct = [
                            int(col) for col in line_list[:6]]
                        local_top.vs4_list.append({'ai': ai, 'aj': aj,
                                                   'ak': ak, 'al': al,
                                                   'am': am, 'funct': funct})
                    elif field == 'position_restraints':
                        # With MARTINI topologie kx, ky, kz can be strings,
                        # so no int() conversions
                        ai, funct, kx, ky, kz = [col for col in line_list[:5]]
                        if ifdef:
                            local_top.if_pos_restr.append(
                                {'def': posre_def, 'ai': int(ai),
                                 'funct': int(funct), 'kx': kx,
                                 'ky': ky, 'kz': kz})
                        else:
                            local_top.pos_restr.append(
                                {'ai': int(ai), 'funct': int(funct), 'kx': kx,
                                 'ky': ky, 'kz': kz})

                    # else:
                    #   raise ValueError('Unknown field : '+field)

        # Needed for empty topologies like aditional ff parameters:
        if local_top_flag:
            self.top_mol_list.append(local_top)

    def write_file(self, itp_file):
        filout = open(itp_file, 'w')
        filout.write("; Itp file created by " + __name__ + "\n")

        for top_mol in self.top_mol_list:
            logger.info(top_mol.name)
            top_mol.write_file(filout)
        for posre in self.posres_file:
            filout.write("\n#ifdef " + posre['def'] + "\n")
            filout.write("#include \"" + posre['file'] + "\"\n")
            filout.write("#endif\n\n")

        filout.close()

    def display(self):
        logger.info('- ITP file: {}'.format(self.name))
        logger.info("- molecules defined in the itp file:")
        for top_mol in self.top_mol_list:
            logger.info("* {}".format(top_mol.name))

    def charge(self, mol_name):
        for top_mol in self.top_mol_list:
            # print(mol_name, top_mol.name)
            if top_mol.name == mol_name:
                return top_mol.get_charge()
        return None

    def res_num(self, mol_name):
        for top_mol in self.top_mol_list:
            # print(mol_name, top_mol.name)
            if top_mol.name == mol_name:
                return top_mol.get_res_num()
        return None

    def add_posre(self, mol_name, posre_name, selec_dict, fc, replace=True):
        for top_mol in self.top_mol_list:
            # print(mol_name, top_mol.name)
            if top_mol.name == mol_name:
                index_posre = top_mol.get_selection_index(
                    selec_dict=selec_dict)
                if index_posre:
                    # Create the posre itp file :
                    logger.debug("Posre for : {}".format(top_mol.name))
                    posre_file_name = os.path.abspath(
                        os.path.join(
                            os_command.get_directory(self.path),
                            self.name + "_posre_" + posre_name + ".itp"))
                    # posre_file_name = self.name+"_posre_"+posre_name+".itp"
                    write_index_posre_file(atom_index_list=index_posre,
                                           posre_file=posre_file_name,
                                           type_val=1, fc=fc)
                    # Add the posre include in the mol itp file:
                    # Need to solve the problem of #ifdef location with .top
                    # files containing self top
                    if replace:
                        # Remove previous def:
                        new_content = ""
                        with open(self.path) as file:
                            posre_def = None
                            for line in file:
                                # print("Itp line: \"{}\" ".format(line))
                                # Check posres include:
                                if line[:6] == '#ifdef':
                                    line_list = line.split()
                                    posre_def = line_list[1]

                                if posre_def != posre_name:
                                    new_content += line

                                if line[:6] == '#endif':
                                    posre_def = None
                        # Add the new one:
                        new_content += '#ifdef ' + posre_name + '\n'
                        new_content += '#include \"' +\
                            os.path.basename(posre_file_name) + "\" \n"
                        new_content += '#endif \n\n'
                        # Save the file:
                        file = open(self.path, "w")
                        file.write(new_content)
                        file.close()

                    else:
                        with open(self.path, 'a') as file:
                            file.write('#ifdef ' + posre_name + '\n')
                            # file.write('#include \"'+self.name+"_posre_"+\
                            # posre_name+".itp\" \n")
                            file.write(
                                '#include \"' +
                                os.path.basename(posre_file_name) + "\" \n")
                            file.write('#endif \n\n')

    def set_top_mol_name(self, new_name):
        if len(self.top_mol_list) == 1:
            self.top_mol_list[0].name = new_name
        else:
            raise ValueError('Cannot set top mol name with multiple top mol '
                             'in itp')

    def get_include_file_list(self):
        file_list = []
        for posre in self.posres_file:
            # print(posre)
            file_list.append(os.path.abspath(os.path.join(
                os_command.get_directory(self.path), posre['file'])))
        return file_list

    def change_mol_name(self, old_name, new_name):

        name_change_flag = False
        for top_mol in self.top_mol_list:
            if top_mol.name == old_name:
                name_change_flag = True
                top_mol.name = new_name
        if name_change_flag:
            self.write_file(self.path)



#!/usr/bin/env python3
# coding: utf-8

"""
Tests for FreeEner class
"""

import pytest
import numpy as np

from gromacs_py.free_ener import FreeEner
from pdb_manip_py import pdb_manip

from .datafiles import PDB_1D30

# Autorship information
__author__ = "Samuel Murail"
__copyright__ = "Copyright 2020, RPBS"
__credits__ = ["Samuel Murail"]
__license__ = "GNU General Public License v2.0"
__maintainer__ = "Samuel Murail"
__email__ = "samuel.murail@u-paris.fr"
__status__ = "Production"


def test_free_bind(tmp_path):
    dna_free = FreeEner('DAP', f'{tmp_path}/1D30_solv')
    dna_free.prepare_complex_pdb(
        PDB_1D30, 'NC(=N)c1ccc(cc1)c2[nH]c3cc(ccc3c2)C(N)=N')

    start_coor = pdb_manip.Coor(dna_free.gmxsys.coor_file)
    assert start_coor.num == 20973
    dna_free.equilibrate_complex(em_steps=100, HA_time=0.001,
                                 CA_time=0.002, CA_LOW_time=0.02,
                                 dt=0.002, dt_HA=0.001, temp=300,
                                 receptor_grp='DNA',
                                 short_steps=500)
    dna_free.compute_add_intermol_from_traj(ref_coor=None, rec_group='DNA')
    dg_rest_water = dna_free.get_water_restr()
    print(dg_rest_water)
    assert ((5.5 < dg_rest_water) and (dg_rest_water < 9.5))

    dna_free.plot_intermol_restr()

    delta_restr = 0.5
    restr_list = np.arange(0, 1 + delta_restr, delta_restr)
    delta_coul = 0.5
    coul_list = np.arange(0, 1 + delta_coul, delta_coul)
    vdw_coul = 0.5
    vdw_list = np.arange(0, 1 + vdw_coul, vdw_coul)
    dna_free.run(lambda_restr_list=restr_list,
                 lambda_coul_list=coul_list,
                 lambda_vdw_list=vdw_list,
                 em_steps=50, nvt_time=0.05, npt_time=0.05, prod_time=0.25,
                 temp_groups='System')
    dna_free.extend_lambda_prod(prod_time=0.5)
    dg_bind, dg_std_bind = dna_free.get_free_ener()
    print(dg_bind, dg_std_bind)
    assert ((0 < dg_bind) and (dg_bind < 200))

    # Need to think about using alchemlyb as a dependance or not
    dna_free.plot_convergence(dt=0.25, graph_out=f'{tmp_path}/alchem.png')
    dna_free.compute_convergence_gbar(dt=0.25)
    dna_free.plot_convergence_graph(graph_out=f'{tmp_path}/bar.png')


@pytest.mark.parametrize("smile, dg", [
    ("c1ccccc1", -1.481),
    ("c1ccccc1O", -0.413),
    ("c1cc(O)ccc1O", -0.826),
])
def test_is_palindrome(smile, dg):
    assert FreeEner.symmetry_correction(smile) == pytest.approx(dg, rel=1e-3)


def test_solv_water_free(tmp_path):
    mol_free = FreeEner('DEN', f'{tmp_path}/indene_water_solv')
    mol_free.water_box_from_SMILE('C1C=Cc2ccccc12')

    start_coor = pdb_manip.Coor(mol_free.gmxsys.coor_file)
    assert start_coor.num == 1625
    # To avoid domain decomposition errors:
    # Use only one proc
    mol_free.gmxsys.nt = 1
    mol_free.equilibrate_solvent_box(em_steps=100, dt=0.002, prod_time=0.02,
                                     short_steps=500)
    delta_coul = 0.5
    coul_list = np.arange(0, 1 + delta_coul, delta_coul)
    vdw_coul = 0.5
    vdw_list = np.arange(0, 1 + vdw_coul, vdw_coul)
    mol_free.run(lambda_coul_list=coul_list,
                 lambda_vdw_list=vdw_list,
                 em_steps=100, nvt_time=0.1, npt_time=0.1, prod_time=0.1,
                 temp_groups='System')
    dg_solv, dg_std_solv = mol_free.get_free_ener()
    print(dg_solv, dg_std_solv)
    assert ((10 < dg_solv) and (dg_solv < 30))


def test_solv_oct(tmp_path):
    mol_free = FreeEner('DEN', f'{tmp_path}/indene_oct_solv')
    mol_free.octanol_box_from_SMILE('C1C=Cc2ccccc12')

    start_coor = pdb_manip.Coor(mol_free.gmxsys.coor_file)
    assert start_coor.num == 1502
    mol_free.gmxsys.nt = 1
    mol_free.equilibrate_solvent_box(em_steps=100, dt=0.002, prod_time=0.02,
                                     short_steps=500)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Create the pipy:
# python setup.py sdist
# twine check dist/*
# twine upload dist/*

# Create conda env:
# conda update conda
# conda env create --name test -f=.conda.yml
# conda activate test
# conda deactivate

from setuptools import setup, find_packages


setup(
    name='gromacs_py',
    version='2.0.0',
    author='Samuel Murail',
    author_email='samuel.murail@u-paris.fr',
    packages=find_packages(),
    scripts=['bin/extend.py',
             'bin/create_peptide.py',
             'bin/create_top.py',
             'bin/prepare_prot.py',
             'bin/insert_mol_no_vmd.py',
             'bin/minimize_pdb.py',
             'bin/solvate_ions.py',
             'bin/prepare_prot_topo_edit.py',
             'bin/equi_3_step.py',
             'bin/production.py',
             'bin/top_em_equi_3_step_prod.py',
             'bin/minimize_pdb_and_cyclic.py'],
    description="Gromacs_py is a python library allowing a simplified use"
                " of the gromacs MD simulation software.",
    long_description=open('README.rst', encoding='utf-8').read(),
    long_description_content_type='text/x-rst',
    url='https://github.com/samuelmurail/gromacs_py',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.5",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
    python_requires='>=3.5',
    install_requires=[
        "pandas",
        "matplotlib",
        "os_command_py==1.1.0",
        "pdb2pqr_htmd_propka30==0.0.4",
        "pdb_manip_py==1.3.9",
        "tqdm",
    ],
    package_data={
        '': ['docs/*rst',
             'docs/*py',
             'docs/notebook/00_basic_example.ipynb'],
        'gromacs_py': ['gmx/template/*mdp',
                       'gmx/template/charmm36-jul2017.ff/*',
                       'test_files/*pdb',
                       'test_files/*xvg'],
    },
    project_urls={
        'Bug Reports': 'https://github.com/samuelmurail/gromacs_py/issues',
        'Funding': 'https://www.impots.gouv.fr/portail/',
        'Source': 'https://github.com/samuelmurail/gromacs_py',
    },
)



#!/usr/bin/env python3

""" Minimize a pdb file structure
"""

import argparse
import shutil
from gromacs_py import gmx
from pdb_manip_py import pdb_manip

__author__ = "Samuel Murail"


def parser_input():

    # Parse arguments :
    parser = argparse.ArgumentParser(
        description='Minimize a cyclic peptide structure in 2 steps, the '
        'first step without bonds constraints and the second step with '
        'bonds constraints')
    parser.add_argument('-f', action="store", dest="f",
                        help='Input PDB file', type=str, required=True)
    parser.add_argument('-n', action="store", dest="name",
                        help='Output file name', type=str, required=True)
    parser.add_argument('-dir', action="store", dest="out_dir",
                        help='Output directory for intermediate files',
                        type=str, default="./tmp_em")
    parser.add_argument('-m_steps', action="store", dest="min_steps",
                        help='Minimisation nsteps, default=1000', type=int,
                        default=1000)
    parser.add_argument('-keep', action="store_true", dest="keep_flag",
                        help='Flag to keep temporary files (without flag '
                        'output directory will be delete')
    parser.add_argument('-cyclic', action="store_true", dest="cyclic_flag",
                        help='Flag to indicate if the peptide/protein is '
                        'cyclic')
    parser.add_argument('-nt', action="store", dest="nt",
                        help='Total number of threads to start, default=0',
                        type=float, default=0)
    parser.add_argument('-ntmpi', action="store", dest="ntmpi",
                        help='Number of thread-MPI threads to start, '
                        'default=0', type=float, default=0)
    parser.add_argument('-gpu_id', action="store", dest="gpuid",
                        help='List of GPU device id-s to use, default=\"\" ',
                        default="None")
    parser.add_argument('-keep_segid', action="store_true", dest="keep_segid",
                        help='Flag to indicate if the original chain/segid '
                        'should be kept')
    parser.add_argument('-add_ter', action="store_true", dest="add_termini",
                        help='Flag to indicate if TER line should be included'
                        ' between chains and if residues in the input pdb '
                        'file have non-consecutive residues')

    return parser


def get_structure_string(pdb_file):
    """Return a pdb_file as a pdb string.
    Add TER line between chains or between
    non consecutive residues.
    """

    str_out = ""
    ref_coor = pdb_manip.Coor(pdb_file)

    _, first_atom = list(sorted(ref_coor.atom_dict.items()))[0]
    chain = first_atom['chain']
    res_num = first_atom['res_num']

    for atom_num, atom in sorted(ref_coor.atom_dict.items()):
        # Atom name should start a column 14, with the type of atom ex:
        #   - with atom type 'C': ' CH3'
        # for 2 letters atom type, it should start at coulumn 13 ex:
        #   - with atom type 'FE': 'FE1'
        name = atom["name"]
        if len(name) <= 3 and name[0] in ['C', 'H', 'O', 'N', 'S', 'P']:
            name = " " + name

        # Add the TER
        if chain != atom["chain"]:
            print('add TER for chain {}'.format(atom['chain']))
            str_out += "TER\n"
            chain = atom["chain"]
            res_num = atom['res_num']
        elif res_num != atom['res_num']:
            if res_num == atom['res_num']-1:
                res_num += 1
            else:
                print('add TER between res {} and {} of chain {}'.format(
                    res_num, atom['res_num'], atom['chain']))
                res_num = atom['res_num']
                str_out += "TER\n"

        # Note : Here we use 4 letter residue name.
        str_out += "{:6s}{:5d} {:4s}{:1s}{:4s}{:1s}{:4d}{:1s}"\
                   "   {:8.3f}{:8.3f}{:8.3f}{:6.2f}{:6.2f}"\
                   "          {:2s}\n".format(
                        atom["field"],
                        atom["num"],
                        name,
                        atom["alter_loc"],
                        atom["res_name"],
                        atom["chain"],
                        atom["res_num"],
                        atom["insert_res"],
                        atom["xyz"][0],
                        atom["xyz"][1],
                        atom["xyz"][2],
                        atom["occ"],
                        atom["beta"],
                        atom['elem'])

    return str_out


def add_ter(pdb_file, pdb_out):
    """ Add TER between chain or
    when residue number are not consecutive.
    """

    coor_str = get_structure_string(pdb_file)

    filout = open(pdb_out, 'w')
    filout.write(coor_str)
    filout.close()


def get_chain_res_list(pdb_file):
    """ Get chain and residue list
    In order to remap them to gromacs
    pdb.
    """

    ref_coor = pdb_manip.Coor(pdb_file)
    ca_coor = ref_coor.select_part_dict(selec_dict={'name': ['CA']})

    chain_res_list = []

    for atom_num, atom in sorted(ca_coor.atom_dict.items()):
        chain_res_list.append([atom['chain'], atom['res_num']])

    return(chain_res_list)


def set_chain_res_list(pdb_file, chain_res_list):
    """ Set the chain and residue as in the provided list.
    Save the new pdb and overwrite the pdb input.
    """

    ref_coor = pdb_manip.Coor(pdb_file)
    i = 0

    for atom_num, atom in sorted(ref_coor.atom_dict.items()):

        if atom['res_num'] == chain_res_list[i][1]:
            atom['chain'] = chain_res_list[i][0]
        elif (i + 1 >= len(chain_res_list)):
            print('Mismatch probably, ligand is included:',
                  chain_res_list[i], atom)
            break
        elif atom['res_num'] == chain_res_list[i + 1][1]:
            i += 1
            atom['chain'] = chain_res_list[i][0]
        else:
            print('Mismatch :', chain_res_list[i],
                  chain_res_list[i + 1], atom)
            print('\n' * 10 + 'WRONG' + '\n' * 10)
            break

    ref_coor.write_pdb(pdb_file, check_file_out=False)


if __name__ == "__main__":

    my_parser = parser_input()
    args = my_parser.parse_args()

    if args.add_termini:
        add_ter(args.f, args.f[:-4] + '_good_ter.pdb')
        input_pdb = args.f[:-4] + '_good_ter.pdb'
    else:
        input_pdb = args.f

    vsite = "none"
    peptide = gmx.GmxSys(name=args.name, coor_file=input_pdb)
    peptide.nt = args.nt
    peptide.ntmpi = args.ntmpi
    if args.gpuid != "None":
        peptide.gpu_id = args.gpuid

    if args.cyclic_flag:
        peptide.cyclic_peptide_top(out_folder=args.out_dir + '/top')
    else:
        peptide.add_top(out_folder=args.out_dir + '/top',
                        pdb2gmx_option_dict={'vsite': 'no',
                                             'ignh': 'yes',
                                             'ter': 'no'})

    peptide.em(out_folder=args.out_dir + '/em', name="min_" + args.name,
               nsteps=args.min_steps, posres="", nstxout=1000,
               create_box_flag=True,
               constraints="none")

    peptide.convert_trj(traj=False)

    # Get the minimised structure:
    shutil.copyfile(peptide.coor_file, args.name + '.pdb')

    if args.keep_segid:
        chain_res_list = get_chain_res_list(input_pdb)
        set_chain_res_list(args.name + '.pdb', chain_res_list)

    # Keep or not the intermediate files:
    if not args.keep_flag:
        shutil.rmtree(args.out_dir, ignore_errors=True)
    else:
        peptide.display()



#!/usr/bin/env python3
# coding: utf-8
##################################
# #######   RTP Class   ##########
##################################


class Rtp:
    """Individual molecule topologie
    """

    def __init__(self, path):
        """Read an itp file and extract [atoms] field
        """
        self.path = path
        self.res_dict = {}
        self.read_file()

        # Then assigne the correct atom charge and type

    def read_file(self):

        field = None
        atom_dict = {}
        bond_list = []
        impr_list = []
        res_name = None

        with open(self.path) as file:
            for line in file:
                line_strip = line.strip()
                if len(line_strip) > 0:
                    if line_strip[0] == "[":
                        # Remove space and [ ], remove also comments
                        field = line.replace(" ", "").split("]")[0][1:]

                        if field not in ['bondedtype', 'atoms', 'bonds',
                                         'impropers', '']:

                            if len(atom_dict) > 0:
                                residue = {}
                                residue['atom'] = atom_dict
                                residue['bond'] = bond_list
                                residue['impr'] = impr_list
                                self.res_dict[res_name] = residue

                            res_name = field
                            atom_dict = {}
                            bond_list = []
                            impr_list = []
                        continue

                    if line_strip[0] != ";" and line_strip[0] != "#":
                        line_list = line_strip.split()

                        if field == 'atoms':
                            atom_name = line_list[0]
                            atom_type = line_list[1]
                            atom_charge = float(line_list[2])
                            charge_group = int(line_list[3])
                            atom_dict[atom_name] = {
                                "type": atom_type,
                                "charge": atom_charge,
                                "charge_group": charge_group}

                        elif field == 'bonds':
                            bond_list.append({'ai': line_list[0],
                                              'aj': line_list[1]})

                        elif field == 'impropers':
                            impr_list.append({'ai': line_list[0],
                                              'aj': line_list[1],
                                              'ak': line_list[2],
                                              'al': line_list[3]})
